//-----------------------------------------------------------------------------
//Text BinaryClock
PROGRAM TextBinaryClock;

//-----------------------------------------------------------------------------
// Import - Modules used in the application
import "mod_key";
import "mod_video";
import "mod_map";
import "mod_text";
import "mod_string";
import "mod_time";
import "mod_proc";


//-----------------------------------------------------------------------------
// Import modules used only for Debugging
//import "mod_say"
//import "mod_debug"

//-----------------------------------------------------------------------------
//include library for the Wiz/Caanoo keys
include "jkeys.lib"

//-----------------------------------------------------------------------------
// Constants
CONST
	version="1.0";          // Application Version

	// ApplicationPhase Enum - Constants used to identify current phase in the program (used to change pages to show)
	AP_None = 0;            // None
	AP_MainPage = 1;        // Show MainPage with Text Binary Clock
	AP_InfoPage = 2;        // Show InfoPage with some About pf the program
	AP_Exit = 3;            // Exit from the program

//-----------------------------------------------------------------------------
// Global Data
GLOBAL
int fontId;                 // ID of the font to use (currently is not loaded a custom font)
int ApplicationPhase;       // Contain current application phase used to change pages to show

//-----------------------------------------------------------------------------
//Declare Types

//Time Type where store single time data
TYPE TimeStruct
	int    Hours;           //Integer value for the Hours
	string HoursString;     //String value for the Hours
	int    Minutes;         //Integer value for the Minutes
	string MinutesString;   //String value for the Hours
	int    Seconds;         //Integer value for the Seconds
	string SecondsString;   //String value for the Hours
END

//Binary structure (contains 8 Bit)
TYPE BinaryStruct
	int Bit[7];
END

//BCD structure used to store decimal and unit values for Hour/Minute/Seconds
TYPE BCDStruct
	BinaryStruct da;
	BinaryStruct u;
END

//-----------------------------------------------------------------------------
//Controller Process - This process is used to coordinate the various processes of the program.
//-----------------------------------------------------------------------------
PRIVATE
	int lastPhase;              // contain the last page show
BEGIN
	set_mode(320,240,16);       // Set video risolution to 320x240 pixels and 16 bits colors
	
	jkeys_set_default_keys();   //Set default keys
	jkeys_controller();         //Start process used to check Wiz/Caanoo keys
	
	ApplicationPhase = AP_MainPage; // Set current phase to MainPage to show Binary Clock
	
	REPEAT
		//-----------------------------------------------------------------------------
		// Check new Phase
		IF(lastPhase != ApplicationPhase)
			//let_me_alone();             //Kill all running process
			SWITCH (ApplicationPhase)
				CASE AP_MainPage:       //Start MainProcess
					MainProcess();
				END
				CASE AP_InfoPage:       //Start InfoProcess
					InfoProcess();
				END
			END
			
			jkeys_controller();         //Start process used to check Wiz/Caanoo keys
		END

		//-----------------------------------------------------------------------------
		// Store current phase
		lastPhase = ApplicationPhase;
		
		//-----------------------------------------------------------------------------
		// Check Keys to switch pages
		IF((key(_i) ||(jkeys_state[_JKEY_SELECT])))

			WaitKeyUp(_i, _JKEY_SELECT);

			//Switch page
			IF(ApplicationPhase == AP_MainPage)
				ApplicationPhase = AP_InfoPage;
			ELSE
				ApplicationPhase = AP_MainPage;
			END
		END

		// Check key to exit
		IF((key(_esc) ||(jkeys_state[_JKEY_MENU])))

			WaitKeyUp(_esc, _JKEY_MENU);
			
			//If in MainPage close the application otherwise back to MainPage
			IF(ApplicationPhase == AP_MainPage)
				ApplicationPhase = AP_Exit;
			ELSE
				ApplicationPhase = AP_MainPage;
			END
		END
		
		FRAME;
	UNTIL (ApplicationPhase == AP_Exit)

	exit();
END

//-----------------------------------------------------------------------------
// Main Process - Show Text Binary Clock
PROCESS MainProcess()
//-----------------------------------------------------------------------------
// Main Process Variables
PRIVATE
int          last_timer;    //Timer used to execute GetTime only 2 times to second
TimeStruct   currentTime;   //Current Time
BCDStruct    bcdHours;      //BCD Struct for the Hours
BCDStruct    bcdMinutes;    //BCD Struct for the Minutes
BCDStruct    bcdSeconds;    //BCD Struct for the Seconds
BinaryStruct binHours;      //BIN Struct for the Hours
BinaryStruct binMinutes;    //BIN Struct for the Minutes
BinaryStruct binSeconds;    //BIN Struct for the Seconds
//-----------------------------------------------------------------------------
BEGIN
	//-----------------------------------------------------------------------------
	//Display information

	write(fontId,8,8,0, "Text Binary Clock v"+ version);
	write(fontId,224,2,0, "by Gigios");
	write(fontId,190,12,0, "www.OpenConsole.it");
	write(fontid,0,16,0,"________________________________________");

	// Draw BCD Text
	DrawHeaderBCD();
	DrawBCDTime(bcdHours, bcdMinutes, bcdSeconds);
	DrawFooter(currentTime, 95);

	// Draw Binary Text
	DrawHeaderBIN();
	DrawBINTime(binHours, binMinutes, binSeconds);
	DrawFooter(currentTime, 200);

	//-----------------------------------------------------------------------------
	//Set last time to force first update
	last_timer = -1000;
	//-----------------------------------------------------------------------------
	//Main Loop
	REPEAT
		// Check timer each 500 milliseconds (+/- 2 times for seconds)
		IF((Get_Timer()-last_timer)>500)
			// Get Current Time
			GetTime(currentTime);

			//-----------------------------------------------------------------------------
			// Convert Time to BCD
			GetBCDValue(bcdHours, currentTime.HoursString);
			GetBCDValue(bcdMinutes, currentTime.MinutesString);
			GetBCDValue(bcdSeconds, currentTime.SecondsString);
			
			//-----------------------------------------------------------------------------
			// Convert Time to Binary
			IntToBinaryStruct(binHours, currentTime.Hours);
			IntToBinaryStruct(binMinutes, currentTime.Minutes);
			IntToBinaryStruct(binSeconds, currentTime.Seconds);
		END

		FRAME;   //Update screen
	UNTIL (ApplicationPhase != AP_MainPage)

	// Remove local text
	delete_text(all_text);
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// InfoPage Process - Show Information
PROCESS InfoProcess()
BEGIN
	//-----------------------------------------------------------------------------
	// Show some information
	write(fontId,8,8,0, "Text Binary Clock v"+ version);
	write(fontId,224,2,0, "by Gigios");
	write(fontId,190,12,0, "www.OpenConsole.it");
	write(fontid,0,16,0,"________________________________________");
	
	// Developers information
	write(fontId,4,36,0,"Simple program used to start learning of BennuGD");
	write(fontId,80,56,0,"From an idea of user Zip");
	write(fontId,66,76,0,"Coding:");
	write(fontId,220,76,0,"Gigios");
	write(fontId,66,86,0,"Ideas and Support:");
	write(fontId,220,86,0,"Zip");
	write(fontId,66,96,0,"Wiz port and Support:");
	write(fontId,220,96,0,"RZZ");
	write(fontId,36,126,0,"We are from the italian community about");
	write(fontId,114,136,0,"Retro Games");
	write(fontId,140,146,0,"and");
	write(fontId,90,156,0,"OpenSource Console");
	write(fontId,46,186,0,"Our community site and WebZine are");
	write(fontId,92,196,0,"www.OpenConsole.it");

	// Description on key to exit
	IF((os_id==OS_GP2X_WIZ) || (os_id==OS_CAANOO))
		write(fontId,46,232,0,"Press Menu to back in the main page.");
	ELSE
		write(fontId,46,232,0,"Press Esc to back in the main page.");
	END
	
	// Wait the key to come back to Main Page
	REPEAT
		FRAME;   //Update screen
	UNTIL (ApplicationPhase != AP_InfoPage)

	// Remove local text
	delete_text(all_text);
END

//-----------------------------------------------------------------------------
// Wait the KeyUp event
//     scanCode = Key to check
//     jKeyCode = jKey to check (console Wiz/Caanoo)
FUNCTION WaitKeyUp(int scanCode, int jKeyCode)
BEGIN
	//Wait Key Up
	REPEAT
		FRAME;
	UNTIL (((!key(scanCode)) && (!jkeys_state[jKeyCode]))|| (ApplicationPhase==AP_Exit))
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Draw Header text for the BCD Binary Clock!
FUNCTION DrawHeaderBCD()
BEGIN
	write(fontId,10,30,0, "BCD");

	DrawHeaderTime(30);                          // Draw Hours/Minutes/Seconds Text

	// Draw "da   u" text for Hours/Minutes/Seconds
	write(fontId,74,50,0, "da");
	write(fontId,108,50,0, "u");
	write(fontId,144,50,0, "da");
	write(fontId,178,50,0, "u");
	write(fontId,214,50,0, "da");
	write(fontId,248,50,0, "u");

	// Draw vertical text (bit number)
	write(fontId,46,60,0, "8 |");
	write(fontId,46,69,0, "4 |");
	write(fontId,46,78,0, "2 |");
	write(fontId,46,87,0, "1 |");
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Get current timer and store data in the TimeStruct TYPE
//     bcdHours (BCDStruct)   = hours to draw.
//     bcdMinutes (BCDStruct) = minutes to draw.
//     bcdSeconds (BCDStruct) = seconds to draw.
FUNCTION DrawBCDTime(BCDStruct bcdHours, BCDStruct bcdMinutes, BCDStruct bcdSeconds)
LOCAL
    int idxBit;
	int yPos;
	int yInc;
BEGIN

	yPos = 60; //Y start position
	yInc = 9;  //Y increment to write next line

	// Loop on all 4 lines of bits to show
	FOR(idxBit=3;idxBit>=0;idxBit--)
		// Hours
		IF(idxBit < 2)
			write_int(fontId,82,yPos,0,&bcdHours.da.Bit[idxBit]);
		END
		write_int(fontId,108,yPos,0,&bcdHours.u.Bit[idxBit]);
		
		// Minutes
		IF(idxBit < 3)
			write_int(fontId,152,yPos,0,&bcdMinutes.da.Bit[idxBit]);
		END
		write_int(fontId,178,yPos,0,&bcdMinutes.u.Bit[idxBit]);
		
		// Seconds
		write_int(fontId,222,yPos,0,&bcdSeconds.da.Bit[idxBit]);
		write_int(fontId,248,yPos,0,&bcdSeconds.u.Bit[idxBit]);
		
		// Increment Y position to move in the next line to draw
		yPos += yInc;
	END
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Draw Footer text and decimal time!
//     currentTime = contains current time to show as decimal value in the footer
//     yPos        = Y position where start to draw clock (100 for BCD, 210 for BIN)
FUNCTION DrawFooter(TimeStruct currentTime, int yPos)
BEGIN
	write(fontId,74,yPos,0,"===========================");

	yPos += 10;                                  // Increment Y position to move in the next line to draw

	write(fontId,10,yPos,0,"decimal:");                              // Decimal text

	write_string(fontId,90,yPos,0,&currentTime.HoursString);         // Decimal hours value
	write_string(fontId,160,yPos,0,&currentTime.MinutesString);      // Decimal minutes value
	write_string(fontId,232,yPos,0,&currentTime.SecondsString);      // Decimal seconds value

	IF((os_id==OS_GP2X_WIZ) || (os_id==OS_CAANOO))
		write(fontId,22,232,0,"Press Select to Information or Menu to exit.");
	ELSE
		write(fontId,40,232,0,"Press I to Information or Esc to exit.");
	END
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Draw Header text for the Binary Clock!
FUNCTION DrawHeaderBIN()
BEGIN
	write(fontId,10,127,0, "BINARY");
	DrawHeaderTime(127);                         // Draw Hours/Minutes/Seconds Text

	// Draw vertical text (bit number)
	write(fontid,60,147,2, "32 |");
	write(fontid,60,156,2, "16 |");
	write(fontid,60,165,2, "8 |");
	write(fontid,60,174,2, "4 |");
	write(fontid,60,183,2, "2 |");
	write(fontid,60,192,2, "1 |");
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Draw Header Hours/Minutes/Seconds
//   yPos        = Y position where start to draw text (100 for BCD, 130 for BIN)
FUNCTION DrawHeaderTime(int yPos)
BEGIN
	// Draw Hours/Minutes/Seconds header
	write(fontId,80,yPos,0, "Hours");
	write(fontId,146,yPos,0, "Minutes");
	write(fontId,214,yPos,0, "Seconds");
	
	yPos += 10;		                             // Increment Y position to move in the next line to draw

	// Draw lines
	write(fontId,74,yPos,0, "-------");
	write(fontId,144,yPos,0, "-------");
	write(fontId,214,yPos,0, "-------");
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Get current timer and store data in the TimeStruct TYPE
//     binHours (BinaryStruct)   = hours to draw.
//     binMinutes (BinaryStruct) = minutes to draw.
//     binSeconds (BinaryStruct) = seconds to draw.
FUNCTION DrawBINTime(BinaryStruct binHours, BinaryStruct binMinutes, BinaryStruct binSeconds)
LOCAL
    int idxBit;
	int yPos;
	int yInc;
BEGIN

	yPos = 147; //Y start position
	yInc = 9;   //Y increment to write next line
	
	// Loop on all 5 lines of bits to show
	FOR(idxBit=5;idxBit>=0;idxBit--)
		// Draw bits
		write_int(fontId,93,yPos,0,&binHours.Bit[idxBit]);
		write_int(fontId,163,yPos,0,&binMinutes.Bit[idxBit]);
		write_int(fontId,235,yPos,0,&binSeconds.Bit[idxBit]);
		
		// Increment Y position to move in the next line to draw
		yPos += yInc;
    END
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Get current timer and store data in the TimeStruct TYPE
//     t (TimeStruct) = structure where return current time
FUNCTION GetTime(TimeStruct * t)
PRIVATE
	string timeString;
BEGIN
	timeString = ftime("%H%M%S",time());         // Get Time in string format ("HHMMSS")

	t.HoursString = substr(timeString,0,2);      // Extract only the hours from the time string
	t.Hours = ITOA(t.HoursString);               // Convert string to integer

	t.MinutesString = substr(timeString,2,2);    // Extract only the minutes from the time string
	t.Minutes = ITOA(t.MinutesString);           // Convert string to integer

	t.SecondsString = substr(timeString,4,2);    // Extract only the seconds from the time string
	t.Seconds = ITOA(t.SecondsString);           // Convert string to integer
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Convert string value (two chars) into bcd structure
//     bcd (BCDStruct)         = structure where return data
//     valueToConvert (string) = value to convert (two chars)
FUNCTION GetBCDValue(BCDStruct * bcd, string valueToConvert)
PRIVATE
	int da;
	int u;
BEGIN
	da = ATOI(substr(valueToConvert, 0, 1));     // Decimal value
	u = ATOI(substr(valueToConvert, 1, 1));      // Unit value
	
	IntToBinaryStruct(bcd.da, da);               // Extract bits
	IntToBinaryStruct(bcd.u, u);                 // Extract bits
END
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Convert integer value into binary structure
//     bin (BinaryStruct)   = structure where return data
//     valueToConvert (int) = value to convert
FUNCTION IntToBinaryStruct(BinaryStruct * bin, int valueToConvert)
PRIVATE
	int n;
	int idxBit;
BEGIN
	n = valueToConvert;                          //Store locally the value to convert
	
	// Reset all 8 bits in the structure
	for(idxBit=0;idxBit<=7;idxBit+=1)
        bin.bit[idxBit] = 0;
    END
	
	// Get all bits in the integer value and store it in the structure
	idxBit = 0;
	WHILE(n >= 1)
		bin.bit[idxBit] = (n mod 2);               //Check the modulus of the division with 2 
		n = n / 2;
		//n = n >> 1;                              //Shift right the value to execute division for 2
		idxBit++;
	END
END
//-----------------------------------------------------------------------------
